/** @file calculate.c
* PURPOSE: file contains calculate function. 
* 
* @author Daniel Hasemann
* @version 1.0
* @date December 16th, 2005
*************************************************/

#include "ndame.h"

/** @brief calculate        
*  
*This function is the core of the programme. It calculates all possible
*solutions to set n queens on n*n fields without attacking themselves. If there
*is a solution, isolutions will be incremented. When singlestep=manual, the
*programme holds its position until pressing a key. With (S) you exit the
*algorithm in singlestep mode and (E) exits the whole programme. When you have
*set save=yes, the solutions will be stored on HDD if theres a valid filename.
*@param *d1 is a struct pointer
*@param irow is an integer
*@see main()
*@author Daniel Hasemann
*@date December 16th, 2005
***************************************/

void calculate(struct data*d1,int irow)
{
    int ii=0;
    static int is;
    char chscan;
    
    if(!irow)
    {
       is=0;                                          //exit clear singlestep for new calculation
    }                                                 
    if(is)
    {
        return;                                       //exit singlestep
    }
    
    while(irow<d1->iblength)
    {
        if(ii<d1->iblength)
        {
            if(!(attack(d1,irow,ii)))                 //checks if queen is attacked
            {
                d1->iboard[irow][ii]=1;               //set queen
                calculate(d1,irow+1);                 //recursion
                d1->iboard[irow][ii]=0;               //clear queen
                
                if(d1->m1==manual)                    //chessboard refresh
                {
                    textcolor(C_CHESSBOARD);
                    if((irow+ii)%2)                   //regenerate board
                    {
                        gotoxy(X_POS+ii,Y_POS+irow);
                        printf("%c",BOARD_PART_1);
                    }
                    else
                    {
                        gotoxy(X_POS+ii,Y_POS+irow);
                        printf("%c",BOARD_PART_2);
                    }
                    textcolor(C_REST);
                }
                ii++;                                 //increment column
            }
            else
            {
                ii++;                                 //increment culumn
            }
        }
        else
        {
            return;
        }
    }
    
    d1->isolutions++;                                 //increment solution
    state(d1,5);                                      //refresh status of solutions
        
    if(d1->isave)                                     //save solution on HDD(*dll file)
    {
        ssolution(d1->iboard,d1->iblength,d1->isolutions,&d1->iauthor,d1->cfilename);
    }
    
    if(d1->m1==manual)                                //Singlestep
    {
        psolution(d1);                                //print solution on screen
        gotoxy(12,10);                                
        printf("(Any Key) - Go on");                  
        gotoxy(9,12);
        printf("(S) - Exit manual Step");
            
        chscan=getch();
        if((chscan=='e')||(chscan=='s'))
        {
            if(chscan=='e')
            {
                exit(1);
            }
            is=1;                                     //when is=1 all calculate functions will closed
        }          
    }
    else
    {
        if(kbhit())                                   //when you press keyboard function 
        {                                             //checks if exit or not
            chscan=getch();
            if(chscan=='e')
            {
                exit(1);
            }
        }             
    }    
}
